import { Component, OnDestroy, OnInit } from '@angular/core';
import { MenuItem, PrimeNGConfig } from 'primeng/api';
import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';
import { TranslateService } from '@ngx-translate/core';
import { GuiService } from 'src/app/service/gui.service';
import { User } from 'src/app/model/auth/user';
import { AuthService } from 'src/app/service/auth.service';
import { MenuElement } from 'src/app/model/view/menu-element';
import { SidebarService } from 'src/app/service/sidebar.service';
import { ConfigService } from 'src/app/service/config.service';
import { UserSettingsService } from 'src/app/service/user_settings.service';
import { SettingsService } from 'src/app/service/settings.service';

@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.scss'],
})
export class HeaderComponent implements OnInit, OnDestroy {
  langList: MenuItem[] = [];
  themeList: MenuItem[] = [];

  userMenuList!: MenuItem[];
  user: User | null = null;
  private unsubscribe$ = new Subject<void>();
  isMobile = window.innerWidth <= 600;

  menu: MenuElement[] = [];

  constructor(
    private translateService: TranslateService,
    private ngConfig: PrimeNGConfig,
    private guiService: GuiService,
    private auth: AuthService,
    private sidebarService: SidebarService,
    private config: ConfigService,
    private settings: SettingsService,
    private userSettings: UserSettingsService
  ) {
    this.guiService.isMobile$
      .pipe(takeUntil(this.unsubscribe$))
      .subscribe(isMobile => {
        this.isMobile = isMobile;
        if (isMobile) {
          this.sidebarService.hide();
        }
      });

    this.auth.user$.pipe(takeUntil(this.unsubscribe$)).subscribe(async user => {
      this.user = user;
      await this.initMenuLists();
      if (user) {
        await this.loadTheme();
        await this.loadLang();
      }
    });

    this.themeList = this.config.settings.themes.map(theme => {
      return {
        label: theme.label,
        command: async () => {
          this.guiService.theme$.next(theme.name);
          await this.userSettings.set('theme', theme.name);
        },
      };
    });
  }

  async ngOnInit() {
    await this.initMenuLists();
  }

  ngOnDestroy() {
    this.unsubscribe$.next();
    this.unsubscribe$.complete();
  }

  async login() {
    await this.auth.login();
  }

  async initMenuLists() {
    await this.initMenuList();
    await this.initLangMenuList();
    await this.initUserMenuList();
  }

  async initMenuList() {
    this.menu = [
      {
        label: 'common.news',
        routerLink: ['/'],
        icon: 'pi pi-home',
      },
      {
        label: 'header.menu.about',
        routerLink: ['/about'],
        icon: 'pi pi-info',
      },
    ];

    if (this.auth.user$.value) {
      this.menu.push({
        label: 'header.menu.admin',
        routerLink: ['/admin'],
        icon: 'pi pi-cog',
        visible: await this.auth.isAdmin(),
      });
    }
  }

  async initLangMenuList() {
    this.langList = [
      {
        label: 'English',
        command: async () => {
          this.translate('en');
          await this.setLang('en');
        },
      },
      {
        label: 'Deutsch',
        command: async () => {
          this.translate('de');
          await this.setLang('de');
        },
      },
    ];
  }

  async initUserMenuList() {
    this.userMenuList = [
      {
        label: `${this.user?.username}`,
        disabled: true,
        visible: !!this.user,
      },
      {
        separator: true,
      },
      {
        label: this.translateService.instant('header.logout'),
        command: async () => {
          await this.auth.logout();
        },
        icon: 'pi pi-sign-out',
      },
    ];
  }

  translate(lang: string) {
    this.translateService.use(lang);
    this.translateService
      .get('primeng')
      .subscribe(res => this.ngConfig.setTranslation(res));
  }

  async loadTheme() {
    const defaultTheme = (await this.settings.get('default_theme')) ?? 'maxlan';
    const userTheme = await this.userSettings.get('theme');
    const theme = userTheme ?? defaultTheme;

    this.guiService.theme$.next(theme);

    if (!userTheme) {
      await this.userSettings.set('theme', theme);
    }
  }

  async loadLang() {
    const defaultLang = (await this.settings.get('default_language')) ?? 'en';
    const userLang = await this.userSettings.get('language');
    const lang = userLang ?? defaultLang;

    this.translate(lang);

    if (userLang) {
      return;
    }
    await this.userSettings.set('language', lang);
  }

  async setLang(lang: string) {
    await this.userSettings.set('language', lang);
  }

  toggleSidebar() {
    if (this.sidebarService.visible$.value) {
      this.sidebarService.hide();
      return;
    }

    this.sidebarService.show();
  }
}
